@@ -14,10 +14,9 @@ Configuration = config.Configuration
 Algorithm = config.Algorithm
 ExportStrategy = config.ExportStrategy
 StatisticsBackend = config.StatisticsBackend
-StoppingCondition = config.StoppingCondition
 TypeInferenceStrategy = config.TypeInferenceStrategy
 
-__version__ = "0.17.0"
+__version__ = "0.20.1"
 __all__ = [
     "set_configuration",
     "run_pynguin",
@@ -26,6 +25,5 @@ __all__ = [
     "Algorithm",
     "ExportStrategy",
     "StatisticsBackend",
-    "StoppingCondition",
     "TypeInferenceStrategy",
 ]
