@@ -26,34 +26,43 @@ import os
 import sys
 import threading
 from pathlib import Path
-from typing import TYPE_CHECKING, Type
+from typing import TYPE_CHECKING
 
-import pynguin.analyses.seeding.initialpopulationseeding as initpopseeding
 import pynguin.assertion.assertiongenerator as ag
 import pynguin.configuration as config
 import pynguin.ga.chromosome as chrom
-import pynguin.ga.chromosomeconverter as cc
 import pynguin.ga.chromosomevisitor as cv
 import pynguin.ga.computations as ff
 import pynguin.ga.postprocess as pp
 import pynguin.ga.testsuitechromosome as tsc
 import pynguin.generation.generationalgorithmfactory as gaf
-import pynguin.testcase.testcase as tc
 import pynguin.utils.statistics.statistics as stat
-from pynguin.analyses.seeding.constantseeding import static_constant_seeding
-from pynguin.generation.export.exportprovider import ExportProvider
+from pynguin.analyses.constants import (
+    ConstantPool,
+    ConstantProvider,
+    DelegatingConstantProvider,
+    DynamicConstantProvider,
+    EmptyConstantProvider,
+    collect_static_constants,
+)
+from pynguin.analyses.module import generate_test_cluster
+from pynguin.analyses.types import TypeInferenceStrategy
+from pynguin.generation import export
 from pynguin.instrumentation.machinery import install_import_hook
-from pynguin.setup.testclustergenerator import TestClusterGenerator
-from pynguin.testcase.execution import ExecutionTracer, TestCaseExecutor
+from pynguin.testcase.execution import (
+    ExecutionTracer,
+    ReturnTypeObserver,
+    TestCaseExecutor,
+)
 from pynguin.utils import randomness
 from pynguin.utils.report import get_coverage_report, render_coverage_report
 from pynguin.utils.statistics.runtimevariable import RuntimeVariable
 
 if TYPE_CHECKING:
+    from pynguin.analyses.module import ModuleTestCluster
     from pynguin.generation.algorithms.testgenerationstrategy import (
         TestGenerationStrategy,
     )
-    from pynguin.setup.testcluster import TestCluster
 
 
 @enum.unique
@@ -100,10 +109,16 @@ def run_pynguin() -> ReturnCode:
         _LOGGER.info("Stop Pynguin Test Generation…")
 
 
-def _setup_test_cluster() -> TestCluster | None:
-    test_cluster = TestClusterGenerator(
-        config.configuration.module_name
-    ).generate_cluster()
+def _setup_test_cluster() -> ModuleTestCluster | None:
+    # TODO this is ugly and needs to be reworked...
+    match config.configuration.type_inference:
+        case config.TypeInferenceStrategy.TYPE_HINTS:
+            strategy = TypeInferenceStrategy.TYPE_HINTS
+        case config.TypeInferenceStrategy.NONE:
+            strategy = TypeInferenceStrategy.NONE
+        case _:
+            strategy = TypeInferenceStrategy.TYPE_HINTS
+    test_cluster = generate_test_cluster(config.configuration.module_name, strategy)
     if test_cluster.num_accessible_objects_under_test() == 0:
         _LOGGER.error("SUT contains nothing we can test.")
         return None
@@ -127,10 +142,17 @@ def _setup_path() -> bool:
     return True
 
 
-def _setup_import_hook() -> ExecutionTracer:
+def _setup_import_hook(
+    dynamic_constant_provider: DynamicConstantProvider | None,
+) -> ExecutionTracer:
     _LOGGER.debug("Setting up instrumentation for %s", config.configuration.module_name)
     tracer = ExecutionTracer()
-    install_import_hook(config.configuration.module_name, tracer)
+
+    install_import_hook(
+        config.configuration.module_name,
+        tracer,
+        dynamic_constant_provider=dynamic_constant_provider,
+    )
     return tracer
 
 
@@ -169,31 +191,46 @@ def _setup_report_dir() -> bool:
 
 def _setup_random_number_generator() -> None:
     """Setup RNG."""
-    if config.configuration.seeding.seed is None:
-        _LOGGER.info("No seed given. Using %d", randomness.RNG.get_seed())
-    else:
-        _LOGGER.info("Using seed %d", config.configuration.seeding.seed)
-        randomness.RNG.seed(config.configuration.seeding.seed)
+    _LOGGER.info("Using seed %d", config.configuration.seeding.seed)
+    randomness.RNG.seed(config.configuration.seeding.seed)
 
 
-def _setup_constant_seeding_collection() -> None:
+def _setup_constant_seeding() -> tuple[
+    ConstantProvider, DynamicConstantProvider | None
+]:
     """Collect constants from SUT, if enabled."""
+    # Use empty provider by default.
+    wrapped_provider: ConstantProvider = EmptyConstantProvider()
+    # We need to return the provider used for dynamic values separately,
+    # because it is later on used to hook up the instrumentation calls.
+    dynamic_constant_provider: DynamicConstantProvider | None = None
     if config.configuration.seeding.constant_seeding:
-        _LOGGER.info("Collecting constants from SUT.")
-        static_constant_seeding.collect_constants(config.configuration.project_path)
+        _LOGGER.info("Collecting static constants from module under test")
+        constant_pool = collect_static_constants(config.configuration.project_path)
+        if len(constant_pool) == 0:
+            _LOGGER.info("No constants found")
+        else:
+            _LOGGER.info("Constants found: %s", len(constant_pool))
+            # Probability of 1.0 -> if a value is requested and available -> return it.
+            wrapped_provider = DelegatingConstantProvider(
+                constant_pool, wrapped_provider, 1.0
+            )
 
-
-def _setup_initial_population_seeding(test_cluster: TestCluster):
-    """Collect and parse tests for seeding the initial population"""
-    if config.configuration.seeding.initial_population_seeding:
-        _LOGGER.info("Collecting and parsing provided testcases.")
-        initpopseeding.initialpopulationseeding.test_cluster = test_cluster
-        initpopseeding.initialpopulationseeding.collect_testcases(
-            config.configuration.seeding.initial_population_data
+    if config.configuration.seeding.dynamic_constant_seeding:
+        _LOGGER.info("Setting up runtime collection of constants")
+        dynamic_constant_provider = DynamicConstantProvider(
+            ConstantPool(),
+            wrapped_provider,
+            config.configuration.seeding.seeded_dynamic_values_reuse_probability,
         )
+        wrapped_provider = dynamic_constant_provider
+
+    return wrapped_provider, dynamic_constant_provider
 
 
-def _setup_and_check() -> tuple[TestCaseExecutor, TestCluster] | None:
+def _setup_and_check() -> tuple[
+    TestCaseExecutor, ModuleTestCluster, ConstantProvider
+] | None:
     """Load the System Under Test (SUT) i.e. the module that is tested.
 
     Perform setup and some sanity checks.
@@ -204,7 +241,8 @@ def _setup_and_check() -> tuple[TestCaseExecutor, TestCluster] | None:
 
     if not _setup_path():
         return None
-    tracer = _setup_import_hook()
+    wrapped_constant_provider, dynamic_constant_provider = _setup_constant_seeding()
+    tracer = _setup_import_hook(dynamic_constant_provider)
     if not _load_sut(tracer):
         return None
     if not _setup_report_dir():
@@ -219,12 +257,10 @@ def _setup_and_check() -> tuple[TestCaseExecutor, TestCluster] | None:
     executor = TestCaseExecutor(tracer)
     _track_sut_data(tracer, test_cluster)
     _setup_random_number_generator()
-    _setup_constant_seeding_collection()
-    _setup_initial_population_seeding(test_cluster)
-    return executor, test_cluster
+    return executor, test_cluster, wrapped_constant_provider
 
 
-def _track_sut_data(tracer: ExecutionTracer, test_cluster: TestCluster) -> None:
+def _track_sut_data(tracer: ExecutionTracer, test_cluster: ModuleTestCluster) -> None:
     """Track data from the SUT.
 
     Args:
@@ -263,7 +299,7 @@ def _track_sut_data(tracer: ExecutionTracer, test_cluster: TestCluster) -> None:
 
 
 def _get_coverage_ff_from_algorithm(
-    algorithm: TestGenerationStrategy, function_type: Type[ff.CoverageFunction]
+    algorithm: TestGenerationStrategy, function_type: type[ff.CoverageFunction]
 ) -> ff.CoverageFunction:
     """Retrieve the coverage function for a test suite of a given coverage type.
 
@@ -287,17 +323,21 @@ def _get_coverage_ff_from_algorithm(
 def _run() -> ReturnCode:
     if (setup_result := _setup_and_check()) is None:
         return ReturnCode.SETUP_FAILED
-    executor, test_cluster = setup_result
+    executor, test_cluster, constant_provider = setup_result
+    # Observe return types during execution.
+    executor.add_observer(ReturnTypeObserver())
 
     algorithm: TestGenerationStrategy = _instantiate_test_generation_strategy(
-        executor, test_cluster
+        executor, test_cluster, constant_provider
     )
     _LOGGER.info("Start generating test cases")
     generation_result = algorithm.generate_tests()
-    if algorithm.stopping_condition.is_fulfilled():
-        _LOGGER.info("Used up all resources (%s).", algorithm.stopping_condition)
-    else:
+    if algorithm.resources_left():
         _LOGGER.info("Algorithm stopped before using all resources.")
+    else:
+        _LOGGER.info("Stopping condition reached")
+        for stop in algorithm.stopping_conditions:
+            _LOGGER.info("%s", stop)
     _LOGGER.info("Stop generating test cases")
 
     # Executions that happen after this point should not influence the
@@ -306,9 +346,10 @@ def _run() -> ReturnCode:
 
     _track_coverage_metrics(algorithm, generation_result)
 
+    # Remove statements after exceptions
+    truncation = pp.ExceptionTruncation()
+    generation_result.accept(truncation)
     if config.configuration.test_case_output.post_process:
-        truncation = pp.ExceptionTruncation()
-        generation_result.accept(truncation)
 
         unused_primitives_removal = pp.TestCasePostProcessor(
             [pp.UnusedStatementsTestCaseVisitor()]
@@ -328,32 +369,23 @@ def _run() -> ReturnCode:
         generation_result.accept(generator)
 
     # Export the generated test suites
-    converter = cc.ChromosomeConverter()
-    generation_result.accept(converter)
-    failing = converter.failing_test_suite
-    passing = converter.passing_test_suite
-    written_to = _export_test_cases(
-        [t.test_case for t in passing.test_case_chromosomes]
-    )
-    _LOGGER.info(
-        "Export %i successful test cases to %s",
-        passing.size(),
-        written_to,
-    )
-    written_to = _export_test_cases(
-        [t.test_case for t in failing.test_case_chromosomes],
-        "_failing",
-        wrap_code=True,
-    )
-    _LOGGER.info("Export %i failing test cases to %s", failing.size(), written_to)
+    if (
+        config.configuration.test_case_output.export_strategy
+        == config.ExportStrategy.PY_TEST
+    ):
+        _export_chromosome(generation_result)
 
     if config.configuration.statistics_output.create_coverage_report:
         render_coverage_report(
-            get_coverage_report(generation_result, executor),
+            get_coverage_report(
+                generation_result,
+                executor,
+                config.configuration.statistics_output.coverage_metrics,
+            ),
             Path(config.configuration.statistics_output.report_dir) / "cov_report.html",
             datetime.datetime.now(),
         )
-    _track_statistics(passing, failing, generation_result)
+    _track_statistics(generation_result)
     _collect_statistics()
     if not stat.write_statistics():
         _LOGGER.error("Failed to write statistics data")
@@ -397,9 +429,13 @@ def _track_coverage_metrics(
 
 
 def _instantiate_test_generation_strategy(
-    executor: TestCaseExecutor, test_cluster: TestCluster
+    executor: TestCaseExecutor,
+    test_cluster: ModuleTestCluster,
+    constant_provider: ConstantProvider,
 ) -> TestGenerationStrategy:
-    factory = gaf.TestSuiteGenerationAlgorithmFactory(executor, test_cluster)
+    factory = gaf.TestSuiteGenerationAlgorithmFactory(
+        executor, test_cluster, constant_provider
+    )
     return factory.get_search_algorithm()
 
 
@@ -419,41 +455,33 @@ def _collect_statistics() -> None:
         stat.set_output_variable_for_runtime_variable(runtime_variable, value)
 
 
-def _track_statistics(
-    passing: chrom.Chromosome,
-    failing: chrom.Chromosome,
-    result: chrom.Chromosome,
-) -> None:
+def _track_statistics(result: chrom.Chromosome) -> None:
     stat.current_individual(result)
     stat.track_output_variable(RuntimeVariable.Size, result.size())
     stat.track_output_variable(RuntimeVariable.Length, result.length())
-    stat.track_output_variable(RuntimeVariable.FailingSize, failing.size())
-    stat.track_output_variable(
-        RuntimeVariable.FailingLength,
-        failing.length(),
-    )
-    stat.track_output_variable(RuntimeVariable.PassingSize, passing.size())
-    stat.track_output_variable(RuntimeVariable.PassingLength, passing.length())
 
 
-def _export_test_cases(
-    test_cases: list[tc.TestCase], suffix: str = "", wrap_code: bool = False
-) -> str:
-    """Export the given test cases.
+def _export_chromosome(
+    chromosome: chrom.Chromosome,
+    file_name_suffix: str = "",
+) -> None:
+    """Export the given chromosome.
 
     Args:
-        test_cases: A list of test cases to export
-        suffix: Suffix that can be added to the file name to distinguish
+        chromosome: the chromosome to export.
+        file_name_suffix: Suffix that can be added to the file name to distinguish
             between different results e.g., failing and succeeding test cases.
-        wrap_code: Whether or not the generated code shall be wrapped
-
     Returns:
         The name of the target file
     """
-    exporter = ExportProvider.get_exporter(wrap_code=wrap_code)
     target_file = os.path.join(
         config.configuration.test_case_output.output_path,
-        "test_" + config.configuration.module_name.replace(".", "_") + suffix + ".py",
+        "test_"
+        + config.configuration.module_name.replace(".", "_")
+        + file_name_suffix
+        + ".py",
     )
-    exporter.export_sequences(target_file, test_cases)
-    return target_file
+    export_visitor = export.PyTestChromosomeToAstVisitor()
+    chromosome.accept(export_visitor)
+    export.save_module_to_file(export_visitor.to_module(), target_file)
+    _LOGGER.info("Written %i test cases to %s", chromosome.size(), target_file)
