@@ -19,18 +19,15 @@ from importlib import reload
 from math import inf
 from queue import Empty, Queue
 from types import CodeType, ModuleType
-from typing import TYPE_CHECKING, Any, Callable
+from typing import TYPE_CHECKING, Any, Callable, TypeVar
 
-import astor
 from bytecode import Compare
 from jellyfish import levenshtein_distance
 from ordered_set import OrderedSet
 
-import pynguin.analyses.controlflow.programgraph as pg
 import pynguin.testcase.statement_to_ast as stmt_to_ast
 import pynguin.utils.namingscope as ns
-from pynguin.analyses.controlflow.cfg import CFG
-from pynguin.analyses.controlflow.controldependencegraph import ControlDependenceGraph
+from pynguin.analyses.controlflow import CFG, ControlDependenceGraph, ProgramGraphNode
 from pynguin.utils.type_utils import (
     given_exception_matches,
     is_bytes,
@@ -133,10 +130,7 @@ class ExecutionContext:
             self._module_aliases, self._variable_names
         )
         statement.accept(visitor)
-        assert (
-            len(visitor.ast_nodes) == 1
-        ), "Expected statement to produce exactly one ast node"
-        return ExecutionContext._wrap_node_in_module(visitor.ast_nodes[0])
+        return ExecutionContext._wrap_node_in_module(visitor.ast_node)
 
     @staticmethod
     def _wrap_node_in_module(node: ast.stmt) -> ast.Module:
@@ -205,6 +199,41 @@ class ExecutionObserver:
         """
 
 
+class ReturnTypeObserver(ExecutionObserver):
+    """Observes the runtime types seen during execution."""
+
+    def __init__(self):
+        self._return_type_trace: dict[int, type] = {}
+
+    def before_test_case_execution(self, test_case: tc.TestCase):
+        self._return_type_trace.clear()
+
+    def after_test_case_execution(
+        self, test_case: tc.TestCase, result: ExecutionResult
+    ):
+        result.store_return_types(dict(self._return_type_trace))
+
+    def before_statement_execution(
+        self, statement: stmt.Statement, exec_ctx: ExecutionContext
+    ):
+        pass  # not relevant
+
+    def after_statement_execution(
+        self,
+        statement: stmt.Statement,
+        exec_ctx: ExecutionContext,
+        exception: Exception | None = None,
+    ) -> None:
+        if (
+            exception is None
+            and (ret_val := statement.ret_val) is not None
+            and not ret_val.is_none_type()
+        ):
+            self._return_type_trace[statement.get_position()] = type(
+                exec_ctx.get_reference_value(ret_val)
+            )
+
+
 class ExecutionResult:
     """Result of an execution."""
 
@@ -213,6 +242,7 @@ class ExecutionResult:
         self._assertion_traces: dict[type, at.AssertionTrace] = {}
         self._execution_trace: ExecutionTrace | None = None
         self._timeout = timeout
+        self._return_type_trace: dict[int, type] = {}
 
     @property
     def timeout(self) -> bool:
@@ -223,6 +253,16 @@ class ExecutionResult:
         """
         return self._timeout
 
+    @property
+    def return_type_trace(self) -> dict[int, type]:
+        """Provide the stored type trace.
+
+        Returns:
+            The observed return types per statement.
+            Not every statement index may have an entry.
+        """
+        return self._return_type_trace
+
     @property
     def exceptions(self) -> dict[int, Exception]:
         """Provide a map of statements indices that threw an exception.
@@ -275,7 +315,7 @@ class ExecutionResult:
         """Returns true if any exceptions were thrown during the execution.
 
         Returns:
-            Whether or not the test has exceptions
+            Whether the test has exceptions
         """
         return bool(self._exceptions)
 
@@ -298,6 +338,58 @@ class ExecutionResult:
             return min(self._exceptions.keys())
         return None
 
+    def store_return_types(self, return_types: dict[int, type]) -> None:
+        """Report that the statement with the given stmt_idx has type tp_.
+
+        Args:
+            return_types: The observed types for each statement index.
+        """
+        self._return_type_trace = return_types
+
+    def delete_statement_data(self, deleted_statements: set[int]) -> None:
+        """It may happen that the test case is modified after execution, for example,
+        by removing unused primitives. We have to update the execution result to reflect
+        this, otherwise the indexes maybe wrong.
+
+        Args:
+            deleted_statements: The indexes of the deleted statements
+        """
+        self._return_type_trace = ExecutionResult.shift_dict(
+            self._return_type_trace, deleted_statements
+        )
+        self._exceptions = ExecutionResult.shift_dict(
+            self._exceptions, deleted_statements
+        )
+
+    T = TypeVar("T")
+
+    @staticmethod
+    def shift_dict(to_shift: dict[int, T], deleted_indexes: set[int]) -> dict[int, T]:
+        """Shifts the entries in the given dictionary by computing their new positions
+        after the given statements were deleted.
+
+        Args:
+            to_shift: The dict to shift
+            deleted_indexes: A set of deleted statement indexes.
+
+        Returns:
+            The shifted dict
+        """
+        # Count how many statements were deleted up to a given point
+        shifts = {}
+        delta = 0
+        for idx in range(max(to_shift.keys(), default=0) + 1):
+            if idx in deleted_indexes:
+                delta += 1
+            shifts[idx] = delta
+
+        # Shift all indexes accordingly
+        shifted = {}
+        for stmt_idx, value in to_shift.items():
+            if stmt_idx not in deleted_indexes:
+                shifted[stmt_idx - shifts[stmt_idx]] = value
+        return shifted
+
     def __str__(self) -> str:
         return (
             f"ExecutionResult(exceptions: {self._exceptions}, "
@@ -316,7 +408,7 @@ class ExecutionTrace:
     executed_predicates: dict[int, int] = field(default_factory=dict)
     true_distances: dict[int, float] = field(default_factory=dict)
     false_distances: dict[int, float] = field(default_factory=dict)
-    covered_lines: OrderedSet[int] = field(default_factory=set)
+    covered_line_ids: OrderedSet[int] = field(default_factory=OrderedSet)
 
     def merge(self, other: ExecutionTrace) -> None:
         """Merge the values from the other trace.
@@ -329,7 +421,7 @@ class ExecutionTrace:
             self.executed_predicates[key] = self.executed_predicates.get(key, 0) + value
         self._merge_min(self.true_distances, other.true_distances)
         self._merge_min(self.false_distances, other.false_distances)
-        self.covered_lines.update(other.covered_lines)
+        self.covered_line_ids.update(other.covered_line_ids)
 
     @staticmethod
     def _merge_min(target: dict[int, float], source: dict[int, float]) -> None:
@@ -371,7 +463,7 @@ class PredicateMetaData:
     code_object_id: int
 
     # The node in the program graph, that defines this predicate.
-    node: pg.ProgramGraphNode
+    node: ProgramGraphNode
 
 
 @dataclass
@@ -722,7 +814,13 @@ class ExecutionTracer:
         Args:
             line_id: the if of the line that was visited
         """
-        self._trace.covered_lines.add(line_id)
+        if threading.current_thread().ident != self._current_thread_identifier:
+            return
+
+        if self._is_disabled():
+            return
+
+        self._trace.covered_line_ids.add(line_id)
 
     def register_line(
         self, code_object_id: int, file_name: str, line_number: int
@@ -770,6 +868,22 @@ class ExecutionTracer:
     def __repr__(self) -> str:
         return "ExecutionTracer"
 
+    def lineids_to_linenos(self, line_ids: OrderedSet[int]) -> OrderedSet[int]:
+        """Convenience method to translate line ids to line numbers.
+
+        Args:
+            line_ids: The ids that should be translated.
+
+        Returns:
+            The line numbers.
+        """
+        return OrderedSet(
+            [
+                self._known_data.existing_lines[line_id].line_number
+                for line_id in line_ids
+            ]
+        )
+
 
 def _eq(val1, val2) -> float:
     """Distance computation for '=='
@@ -1025,7 +1139,9 @@ class TestCaseExecutor:
                 self._before_test_case_execution(test_case)
                 return_queue: Queue = Queue()
                 thread = threading.Thread(
-                    target=self._execute_test_case, args=(test_case, return_queue)
+                    target=self._execute_test_case,
+                    args=(test_case, return_queue),
+                    daemon=True,
                 )
                 thread.start()
                 thread.join(timeout=len(test_case.statements))
@@ -1100,13 +1216,13 @@ class TestCaseExecutor:
     ) -> Exception | None:
         ast_node = exec_ctx.executable_node_for(statement)
         if self._logger.isEnabledFor(logging.DEBUG):
-            self._logger.debug("Executing %s", astor.to_source(ast_node))
+            self._logger.debug("Executing %s", ast.unparse(ast_node))
         code = compile(ast_node, "<ast>", "exec")
         try:
             # pylint: disable=exec-used
             exec(code, exec_ctx.global_namespace, exec_ctx.local_namespace)  # nosec
         except Exception as err:  # pylint: disable=broad-except
-            failed_stmt = astor.to_source(ast_node)
+            failed_stmt = ast.unparse(ast_node)
             TestCaseExecutor._logger.debug(
                 "Failed to execute statement:\n%s%s", failed_stmt, err.args
             )
